# coding: utf-8
from django import template
from shipping.models import Country

register = template.Library()


@register.inclusion_tag('shipping/freight.html', takes_context=True)
def shipping_freight(context):
    countries = Country.objects.filter(zone__status=1).filter(status=1)\
        .filter(zone__isnull=False).distinct().order_by('name')

    context.update({'countries': countries})

    return context



# coding: utf-8
from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from shipping.packing.package import Package
from shipping.packing import binpack
from shipping.carriers.correios import CorreiosInterface, CorreiosService
from shipping.carriers.upsinterface import UPSInterface


STATUS = (
    (0, 'Unavailable'),
    (1, 'Available')
)


class Zone(models.Model):
    name = models.CharField(max_length=100)
    status = models.IntegerField(max_length=2, choices=STATUS)
    carrier = models.ForeignKey('Carrier', null=True)

    def get_carrier(self):
        """ get the appropriate carrier class
        """
        carriers_class = ('correioscarrier', 'upscarrier')

        for carrier_class in carriers_class:
            try:
                return getattr(self.carrier, carrier_class)
            except ObjectDoesNotExist:
                pass

    def __unicode__(self):
        return self.name


class Country(models.Model):
    name = models.CharField(max_length=100)
    iso = models.CharField(max_length=20)
    status = models.IntegerField(max_length=2, choices=STATUS)
    zone = models.ForeignKey(Zone)

    @property
    def needs_full_address(self):
        return self.zone.get_carrier().needs_full_address

    def __unicode__(self):
        return self.name


class State(models.Model):
    name = models.CharField(max_length=100)
    iso = models.CharField(max_length=20)
    country = models.ForeignKey(Country, null=True, related_name='states')

    def __unicode__(self):
        return self.name


class Bin(models.Model):
    name = models.CharField(max_length=255)
    width = models.FloatField(help_text="width in centimeters")
    height = models.FloatField(help_text="height in centimeters")
    length = models.FloatField(help_text="length in centimeters")
    weight = models.FloatField(help_text="peso in kilograms")

    carrier = models.ForeignKey('Carrier', related_name='bins')

    def get_package(self):
        return Package((self.height, self.width, self.length))

    def __unicode__(self):
        if self.width and self.height and self.length:
            return "{name} ({w}x{h}x{l})".format(name=self.name, w=self.width,
                h=self.height, l=self.length)

        return self.name


class Carrier(models.Model):
    name = models.CharField(max_length=100)
    status = models.IntegerField(max_length=2, choices=STATUS)

    def estimate_shipping(self, dimensions, country, state, zipcode):
        """ method that finds optimal solution for packing the products
        and according to the zone's carrier calc shipping estimation to zipcode

        :Parameter
          - dimensions: a list of product's dimension '{height}x{width}x{length}x{weight}'
          - zipcode: a valid zip code
          - state: a valid state
        """
        packages = []
        for dimension in dimensions:
            height, width, length, weight = dimension.split('x')
            size = (float(height), float(width), float(length))

            package = Package(size, weight=float(weight))
            packages.append(package)

        best_bin = self.get_best_bin_for_packages(packages)

        if not best_bin:
            raise ValueError('This carrier does not have a valid bin')

        # calc the best packing
        best_packing, rest = binpack(packages, best_bin.get_package())

        total_cost = self.interface.get_shipping_cost(
                bin=best_bin, packages=best_packing, country=country, state=state, zipcode=zipcode)

        return total_cost

    def get_best_bin_for_packages(self, packages):
        """ choose the best bin for a list of packages
        """
        my_packages = []
        for bin in self.bins.all():
            package = bin.get_package()
            package.bin = bin
            my_packages.append(package)

        greater_package = max(packages)
        my_packages.sort()

        for package in my_packages:
            if package.heigth > greater_package.heigth:
                return package.bin

        # when best bin not found, returns de greater bin
        if my_packages:
            return max(my_packages).bin

    def __unicode__(self):
        return self.name


class UPSCarrier(Carrier):
    WEIGHT_UNITS = (
        ('kg', 'kilograms'),
        ('lb', 'pounds'),
    )
    DIMENSION_UNITS = (
        ('cm', 'centimeters'),
        ('in', 'inches'),
    )

    # general
    ups_login = models.CharField(max_length=255, null=True)
    ups_password = models.CharField(max_length=255, null=True)
    ups_id = models.CharField(max_length=255, null=True)
    ups_api_key = models.CharField(max_length=255, null=True)

    # local confs
    weight_unit = models.CharField(max_length=3, choices=WEIGHT_UNITS, default='kg')
    dimension_unit = models.CharField(max_length=3, choices=DIMENSION_UNITS, default='cm')

    # sender address
    address_line_1 = models.CharField(max_length=255, null=True)
    address_line_2 = models.CharField(max_length=255, null=True)
    zip_code = models.CharField(max_length=20, null=True)
    city = models.CharField(max_length=255, null=True)
    country = models.ForeignKey(Country, null=True)
    state = models.ForeignKey(State, null=True)

    # services
    RATE_SERVICES = (
        (1, "United States Domestic Shipments"),
        (2, "Shipments Originating in United States"),
        (3, "Shipments Originating in Puerto Rico"),
        (4, "Shipments Originating in Canada"),
        (5, "Shipments Originating in Mexico"),
        (6, "Polish Domestic Shipments"),
        (7, "Shipments Originating in the European Union"),
        (8, "Shipments Originating in Other Countries"),
    )
    rate_service = models.IntegerField(max_length=2, choices=RATE_SERVICES, default=8)

    PICKUP_TYPES = (
        (1, "Daily Pickup"),
        (3, "Customer Counter"),
        (6, "One Time Pickup"),
        (7, "On Call Air"),
        (11, "Suggested Retail Rates"),
        (19, "Letter Center"),
        (20, "Air Service Center")
    )
    pickup_type = models.IntegerField(max_length=2, choices=PICKUP_TYPES, null=True)

    PACKAGES_TYPES = [
        ('02', 'Custom Packaging'),
        ('01', 'UPS Letter'),
        ('03', 'Tube'),
        ('04', 'PAK'),
        ('21', 'UPS Express Box'),
        ('2a', 'Small Express Box'),
        ('2b', 'Medium Express Box'),
        ('2c', 'Large Express Box'),
    ]
    package_type = models.CharField(max_length=3, choices=PACKAGES_TYPES, default='21')

    @property
    def needs_full_address(self):
        return True

    @property
    def interface(self):
        return UPSInterface(self)


class CorreiosCarrier(Carrier):
    correios_company = models.CharField(max_length=200, null=True, help_text='required when using E-Sedex')
    correios_password = models.CharField(max_length=200, null=True, help_text='required when using E-Sedex')
    zip_code = models.CharField(max_length=20, null=True)

    @property
    def needs_full_address(self):
        return False

    @property
    def interface(self):
        return CorreiosInterface(zip_from=self.zip_code, company=self.correios_company,
            password=self.correios_password, service=CorreiosService.SEDEX)



# coding: utf-8
import json
import logging
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_protect
from django.http import HttpResponse, Http404
from shipping.models import Country, State


def countries(request):
    countries = Country.objects.filter(zone__status=1).filter(status=1)\
        .order_by('name')

    response = {'countries': []}
    for country in countries:
        response['countries'].append({'iso': country.iso, 'name': country.name})

    return HttpResponse(json.dumps(response), mimetype="application/json;charset=utf-8")


def states(request, country_code):
    country = get_object_or_404(Country, iso=country_code)
    states = country.states.order_by('name').all()

    response = {'states': []}
    for state in states:
        response['states'].append({'iso': state.iso, 'name': state.name, 'id': state.id})

    return HttpResponse(json.dumps(response), mimetype="application/json;charset=utf-8")


@csrf_protect
def estimation(request):
    dimensions = request.POST.getlist('dimensions')
    zipcode = request.POST.get('zipcode')
    state = request.POST.get('state')

    country_code = request.POST.get('country_code')

    country = get_object_or_404(Country, iso=country_code)
    if state:
        state = get_object_or_404(State, id=state)

    if len(dimensions) < 1:
        raise Http404()

    try:
        carrier = country.zone.get_carrier()
        price = carrier.estimate_shipping(dimensions, country, state, zipcode)

        response = json.dumps({'price': price})
    except:
        logging.exception('oops, problem when estimate shipping')
        response = json.dumps({'error': "your country's carrier is unavailable"})

    return HttpResponse(response, mimetype="application/json;charset=utf-8")



# coding: utf-8
"""
This code implemnts 3D bin packing in pure Python

Bin packing in this context is calculating the best way to store a number of differently sized boxes in a
number of fixed sized "bins". It is what usually happens in a Warehouse bevore shipping.

The Algorithm has a simple fit first approach, but can archive relative good results because it tries
different rectangular rotations of the packages. Since the Algorithm can't interate over all possible
combinations we use a heuristic approach.

For a few dozen packages it reaches adaequate runtime. Below are the results calculated about a set of
500 real world packing problems.

Binsize     Runtime                 Recuction in shipped Packages
600x400x400 31.5993559361 4970 2033 40.9054325956
600x445x400 31.5596890450 4970 1854 37.3038229376
600x500x400 29.1432909966 4970 1685 33.9034205231


On the datasets we operate on we can archive comparable preformance to academic higly optimized C code
like David Pisinger's 3bpp:

     Runtime                 Recuction in shipped Packages
py   11.3468761444 2721 1066 39.1767732451
3bpp 9.95857691765 2721 1086 39.9117971334

The Python implementation is somewhat slower but can archive slightly better packing results on our
datasets.


Created by Maximillian Dornseif on 2010-08-14.
Copyright (c) 2010 HUDORA. All rights reserved.

https://github.com/hudora/pyShipping
"""
import random
from shipping.packing.package import Package


def packstrip(bin, p):
    """Creates a Strip which fits into bin.

    Returns the Packages to be used in the strip, the dimensions of the strip
    as a 3-tuple and a list of "left over" packages.
    """
    # This code is somewhat optimized and somewhat unreadable
    s = []                # strip
    r = []                # rest
    ss = sw = sl = 0      # stripsize
    bs = bin.heigth       # binsize
    sapp = s.append       # speedup
    rapp = r.append       # speedup
    ppop = p.pop          # speedup
    while p and (ss <= bs):
        n = ppop(0)
        nh, nw, nl = n.size
        if ss + nh <= bs:
            ss += nh
            sapp(n)
            if nw > sw:
                sw = nw
            if nl > sl:
                sl = nl
        else:
            rapp(n)
    return s, (ss, sw, sl), r + p


def packlayer(bin, packages):
    strips = []
    layersize = 0
    layerx = 0
    layery = 0
    binsize = bin.width
    while packages:
        strip, (sizex, stripsize, sizez), rest = packstrip(bin, packages)
        if layersize + stripsize <= binsize:
            packages = rest
            if not strip:
                # we were not able to pack anything
                break
            layersize += stripsize
            layerx = max([sizex, layerx])
            layery = max([sizez, layery])
            strips.extend(strip)
        else:
            # Next Layer please
            packages = strip + rest
            break
    return strips, (layerx, layersize, layery), packages


def packbin(bin, packages):
    packages.sort()
    layers = []
    contentheigth = 0
    contentx = 0
    contenty = 0
    binsize = bin.length
    while packages:
        layer, (sizex, sizey, layersize), rest = packlayer(bin, packages)
        if contentheigth + layersize <= binsize:
            packages = rest
            if not layer:
                # we were not able to pack anything
                break
            contentheigth += layersize
            contentx = max([contentx, sizex])
            contenty = max([contenty, sizey])
            layers.extend(layer)
        else:
            # Next Bin please
            packages = layer + rest
            break
    return layers, (contentx, contenty, contentheigth), packages


def packit(bin, originalpackages):
    packedbins = []
    packages = sorted(originalpackages)
    while packages:
        packagesinbin, (binx, biny, binz), rest = packbin(bin, packages)
        if not packagesinbin:
            # we were not able to pack anything
            break
        packedbins.append(packagesinbin)
        packages = rest
    # we now have a result, try to get a better result by rotating some bins

    return packedbins, rest


# In newer Python versions these van be imported:
# from itertools import permutations
def product(*args, **kwds):
    # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
    # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
    pools = map(tuple, args) * kwds.get('repeat', 1)
    result = [[]]
    for pool in pools:
        result = [x + [y] for x in result for y in pool]
    for prod in result:
        yield tuple(prod)


def permutations(iterable, r=None):
    pool = tuple(iterable)
    n = len(pool)
    r = n if r is None else r
    for indices in product(range(n), repeat=r):
        if len(set(indices)) == r:
            yield tuple(pool[i] for i in indices)


class Timeout(Exception):
    pass


def allpermutations_helper(permuted, todo, maxcounter, callback, bin, bestpack,
    counter):

    if not todo:
        return counter + callback(bin, permuted, bestpack)
    else:
        others = todo[1:]
        thispackage = todo[0]
        for dimensions in set(permutations((thispackage[0], thispackage[1],
            thispackage[2]))):

            thispackage = Package(dimensions, nosort=True)
            if thispackage in bin:
                counter = allpermutations_helper(permuted + [thispackage], others, maxcounter, callback,
                                                 bin, bestpack, counter)
            if counter > maxcounter:
                raise Timeout('more than %d iterations tries' % counter)
        return counter


def trypack(bin, packages, bestpack):
    bins, rest = packit(bin, packages)
    if len(bins) < bestpack['bincount']:
        bestpack['bincount'] = len(bins)
        bestpack['bins'] = bins
        bestpack['rest'] = rest
    if bestpack['bincount'] < 2:
        raise Timeout('optimal solution found')
    return len(packages)


def allpermutations(todo, bin, iterlimit=5000):
    random.seed(1)
    random.shuffle(todo)
    bestpack = dict(bincount=len(todo) + 1)
    try:
        # First try unpermuted
        trypack(bin, todo, bestpack)
        # now try permutations
        allpermutations_helper([], todo, iterlimit, trypack, bin, bestpack, 0)
    except Timeout:
        pass
    return bestpack['bins'], bestpack['rest']


def binpack(packages, bin, iterlimit=5000):
    """Packs a list of Package() objects into a number of equal-sized bins.

    Returns a list of bins listing the packages within the bins and a list of packages which can't be
    packed because they are to big."""
    return allpermutations(packages, bin, iterlimit)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import DataMigration
from shipping.models import UPSCarrier, CorreiosCarrier, Bin


class Migration(DataMigration):

    def forwards(self, orm):
        carrier = UPSCarrier.objects.create(name='UPS', status=1)

        # Large-18" x 13" x 3"
        Bin.objects.create(name='UPS Express Box - Large',
            height=45.72, width=33.02, length=7.62, weight=0,
            carrier=carrier)

        # Medium-15" x 11" x 3"
        Bin.objects.create(name='UPS Express Box - Medium',
            height=38.1, width=27.94, length=7.62, weight=0,
            carrier=carrier)

        # Small- 13" x 11" x 2"
        Bin.objects.create(name='UPS Express Box - Small',
            height=33.02, width=27.94, length=5.08, weight=0,
            carrier=carrier)

        carrier = CorreiosCarrier.objects.create(name='Correios',
            status=1)

    def backwards(self, orm):
        db.execute_many('delete * from carrier;')
        db.execute_many('delete * from bin;')

    complete_apps = ['shipping']



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('shipping.views',

    url(r'countries.json$', 'countries', name='shipping-countries'),
    url(r'countries/(?P<country_code>.+)\.json$', 'states', name='shipping-states'),

    url(r'estimation/?$', 'estimation', name='shipping-estimation'),
)



# coding: utf-8
from django.contrib import admin
from shipping.models import (Zone, Country, State, UPSCarrier, CorreiosCarrier, Bin)


class ZoneAdmin(admin.ModelAdmin):
    list_display = ('name', 'status')
    list_filter = ('status', )


class CountryAdmin(admin.ModelAdmin):
    list_display = ('name', 'iso', 'status')
    list_filter = ('zone', 'status')
    search_fields = ('name', )


class StateAdmin(admin.ModelAdmin):
    list_display = ('name', 'country')
    list_filter = ('country__zone', )
    search_fields = ('name', 'country__name')


class BinInline(admin.TabularInline):
    model = Bin
    extra = 5


class CorreiosCarrierAdmin(admin.ModelAdmin):
    inlines = [BinInline]

admin.site.register(Zone, ZoneAdmin)
admin.site.register(Country, CountryAdmin)
admin.site.register(State, StateAdmin)
admin.site.register(UPSCarrier)
admin.site.register(CorreiosCarrier, CorreiosCarrierAdmin)
admin.site.register(Bin)



# -*- coding: utf-8 -*-
import os
from south.db import db
from south.v2 import DataMigration


class Migration(DataMigration):

    def forwards(self, orm):
        path = lambda p: os.path.join(os.path.dirname(__file__), p)

        db.execute_many(open(path('zone.sql')).read())
        db.execute_many(open(path('country.sql')).read())
        db.execute_many(open(path('state.sql')).read())

    def backwards(self, orm):
        db.execute_many('delete * from shipping_zone;')
        db.execute_many('delete * from shipping_country;')
        db.execute_many('delete * from state;')

    complete_apps = ['shipping']



# coding: utf-8
from ups.client import UPSClient
from ups.model import Address, Package as UPSPackage


class UPSInterface(object):

    def __init__(self, ups_carrier):

        self.credentials = {
            'username': ups_carrier.ups_login,
            'password': ups_carrier.ups_password,
            'access_license': ups_carrier.ups_api_key,
            'shipper_number': ups_carrier.ups_id,
        }
        self.shipper = Address(name='shipper address name', city=ups_carrier.city,
            address=ups_carrier.address_line_1, state=ups_carrier.state.iso,
            zip=ups_carrier.zip_code, country=ups_carrier.country.iso,
            address2=ups_carrier.address_line_2)
        self.package_type = ups_carrier.package_type

    def get_shipping_cost(self, bin, packages, country, state, zipcode):

        ups_packages = []
        for pack in packages:
            weight_total = sum([package.weight for package in pack]) + bin.weight

            ups_packages.append(UPSPackage(weight=weight_total, length=bin.length,
            width=bin.width, height=bin.height))

        recipient = Address(name='recipient address name', city='',
            address='', state=state.iso, zip=zipcode, country=country.iso)

        ups = UPSClient(self.credentials)
        rate_result = ups.rate(ups_packages, self.shipper, recipient, self.package_type)

        return rate_result['info'][0]['cost']



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Zone'
        db.create_table('shipping_zone', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('status', self.gf('django.db.models.fields.IntegerField')(max_length=2)),
            ('carrier', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['shipping.Carrier'], null=True)),
        ))
        db.send_create_signal('shipping', ['Zone'])

        # Adding model 'Country'
        db.create_table('shipping_country', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('iso', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('status', self.gf('django.db.models.fields.IntegerField')(max_length=2)),
            ('zone', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['shipping.Zone'])),
        ))
        db.send_create_signal('shipping', ['Country'])

        # Adding model 'State'
        db.create_table('shipping_state', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('iso', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['shipping.Country'], null=True)),
        ))
        db.send_create_signal('shipping', ['State'])

        # Adding model 'Bin'
        db.create_table('shipping_bin', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('width', self.gf('django.db.models.fields.FloatField')()),
            ('height', self.gf('django.db.models.fields.FloatField')()),
            ('length', self.gf('django.db.models.fields.FloatField')()),
            ('weight', self.gf('django.db.models.fields.FloatField')()),
            ('carrier', self.gf('django.db.models.fields.related.ForeignKey')(related_name='bins', to=orm['shipping.Carrier'])),
        ))
        db.send_create_signal('shipping', ['Bin'])

        # Adding model 'Carrier'
        db.create_table('shipping_carrier', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('status', self.gf('django.db.models.fields.IntegerField')(max_length=2)),
        ))
        db.send_create_signal('shipping', ['Carrier'])

        # Adding model 'UPSCarrier'
        db.create_table('shipping_upscarrier', (
            ('carrier_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['shipping.Carrier'], unique=True, primary_key=True)),
            ('ups_login', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('ups_password', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('ups_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('ups_api_key', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('weight_unit', self.gf('django.db.models.fields.CharField')(default='kg', max_length=3)),
            ('dimension_unit', self.gf('django.db.models.fields.CharField')(default='cm', max_length=3)),
            ('address_line_1', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('address_line_2', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('zip_code', self.gf('django.db.models.fields.CharField')(max_length=20, null=True)),
            ('city', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['shipping.Country'], null=True)),
            ('state', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['shipping.State'], null=True)),
            ('rate_service', self.gf('django.db.models.fields.IntegerField')(default=8, max_length=2)),
            ('pickup_type', self.gf('django.db.models.fields.IntegerField')(max_length=2, null=True)),
            ('package_type', self.gf('django.db.models.fields.CharField')(default='21', max_length=3)),
        ))
        db.send_create_signal('shipping', ['UPSCarrier'])

        # Adding model 'CorreiosCarrier'
        db.create_table('shipping_correioscarrier', (
            ('carrier_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['shipping.Carrier'], unique=True, primary_key=True)),
            ('correios_company', self.gf('django.db.models.fields.CharField')(max_length=200, null=True)),
            ('correios_password', self.gf('django.db.models.fields.CharField')(max_length=200, null=True)),
            ('zip_code', self.gf('django.db.models.fields.CharField')(max_length=20, null=True)),
        ))
        db.send_create_signal('shipping', ['CorreiosCarrier'])


    def backwards(self, orm):
        # Deleting model 'Zone'
        db.delete_table('shipping_zone')

        # Deleting model 'Country'
        db.delete_table('shipping_country')

        # Deleting model 'State'
        db.delete_table('shipping_state')

        # Deleting model 'Bin'
        db.delete_table('shipping_bin')

        # Deleting model 'Carrier'
        db.delete_table('shipping_carrier')

        # Deleting model 'UPSCarrier'
        db.delete_table('shipping_upscarrier')

        # Deleting model 'CorreiosCarrier'
        db.delete_table('shipping_correioscarrier')


    models = {
        'shipping.bin': {
            'Meta': {'object_name': 'Bin'},
            'carrier': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bins'", 'to': "orm['shipping.Carrier']"}),
            'height': ('django.db.models.fields.FloatField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'length': ('django.db.models.fields.FloatField', [], {}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'weight': ('django.db.models.fields.FloatField', [], {}),
            'width': ('django.db.models.fields.FloatField', [], {})
        },
        'shipping.carrier': {
            'Meta': {'object_name': 'Carrier'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'status': ('django.db.models.fields.IntegerField', [], {'max_length': '2'})
        },
        'shipping.correioscarrier': {
            'Meta': {'object_name': 'CorreiosCarrier', '_ormbases': ['shipping.Carrier']},
            'carrier_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['shipping.Carrier']", 'unique': 'True', 'primary_key': 'True'}),
            'correios_company': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'correios_password': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'})
        },
        'shipping.country': {
            'Meta': {'object_name': 'Country'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'status': ('django.db.models.fields.IntegerField', [], {'max_length': '2'}),
            'zone': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shipping.Zone']"})
        },
        'shipping.state': {
            'Meta': {'object_name': 'State'},
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shipping.Country']", 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'iso': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'shipping.upscarrier': {
            'Meta': {'object_name': 'UPSCarrier', '_ormbases': ['shipping.Carrier']},
            'address_line_1': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'address_line_2': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'carrier_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['shipping.Carrier']", 'unique': 'True', 'primary_key': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shipping.Country']", 'null': 'True'}),
            'dimension_unit': ('django.db.models.fields.CharField', [], {'default': "'cm'", 'max_length': '3'}),
            'package_type': ('django.db.models.fields.CharField', [], {'default': "'21'", 'max_length': '3'}),
            'pickup_type': ('django.db.models.fields.IntegerField', [], {'max_length': '2', 'null': 'True'}),
            'rate_service': ('django.db.models.fields.IntegerField', [], {'default': '8', 'max_length': '2'}),
            'state': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shipping.State']", 'null': 'True'}),
            'ups_api_key': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'ups_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'ups_login': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'ups_password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'weight_unit': ('django.db.models.fields.CharField', [], {'default': "'kg'", 'max_length': '3'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'})
        },
        'shipping.zone': {
            'Meta': {'object_name': 'Zone'},
            'carrier': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shipping.Carrier']", 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'status': ('django.db.models.fields.IntegerField', [], {'max_length': '2'})
        }
    }

    complete_apps = ['shipping']


from setuptools import setup
import os


def get_packages():
    # setuptools can't do the job :(
    packages = []
    for root, dirnames, filenames in os.walk('shipping'):
        if '__init__.py' in filenames:
            packages.append(".".join(os.path.split(root)).strip("."))

    return packages


setup(
    name='django-shipping',
    version='0.0.3',
    description='Django shipping tool',
    long_description=open("README.md").read(),
    author=u'Marcel Nicolay',
    author_email='marcelnicolay@gmail.com',
    url='http://github.com/quatix/django-shipping',
    install_requires=open("requirements.txt").read().split("\n"),
    packages=get_packages(),
    package_data={'shipping': ['templates/**/*.html', 'static/**/*/*', 'migrations/*.sql']}
)



#coding: utf-8
import logging
import urllib2
import urllib
import simplexml
import math

logger = logging.getLogger(__name__)


class CorreiosFormat:
    """enumerate available correios formats
    """
    PACOTE = 1
    ROLO = 2


class CorreiosService:
    """enumerate available correios services
    """
    PAC = '41106'
    SEDEX = '40010'
    SEDEX10 = '40215'
    SEDEXHOJE = '40290'
    SEDEXCOBRAR = '40045'


class CorreiosInterface(object):
    """implements basic integration with federal brazilian carrier, Correios.
    """
    _endpoint = 'http://ws.correios.com.br/calculador/CalcPrecoPrazo.aspx'
    _min_dimensions = {
        CorreiosFormat.PACOTE: (18, 5, 0, 0),
        CorreiosFormat.ROLO: (16, 0, 2, 5)
    }

    def __init__(self, zip_from, company=None, password=None,
        service=CorreiosService.SEDEX, format=CorreiosFormat.PACOTE):
        """create a new instance of correios carrier

        :Parameters
          - zip_from: zipcode of where package will be delivered
          - password: correios company password
          - company: correios company code
          - service: correios service
          - format: format of package
        """
        self._zip_from = zip_from
        self._password = password
        self._company = company
        self._service = service
        self._format = format

        self._length = None
        self._diameter = None
        self._height = None
        self._width = None
        self._zip_to = None

        self._aviso_recebimento = 'N'
        self._valor_declarado = 0
        self._mao_propria = 'N'

    def _get_diameter(self):
        """returns the diameter of the sphere inscrit in the package
        """
        return int(math.sqrt(math.pow(self._length, 2) +
            math.pow(self._width, 2)))

    def _set_dimensions(self, package, weight_total):
        self._length, self._diameter, self._height, self._width = self._min_dimensions.get(self._format)
        self._weight = 0.3

        if package.height > self._height:
            self._height = package.height

        if package.width > self._width:
            self._width = package.width

        if package.length > self._length:
            self._length = package.length

        if weight_total > self._weight:
            self._weight = weight_total

        # height can't be greater than length
        if self._height > self._length:
            self._height = self._length

        # width can't be less than 11cm when length is less than 25cm
        if self._format == CorreiosFormat.PACOTE and self._width < 11 \
            and self._length < 25:
            self._width = 11

        # set diameter
        diameter = self._get_diameter()
        if diameter > self._diameter:
            self._diameter = diameter

    def _get_parameters(self):
        params = {
            'nCdEmpresa': self._company,
            'sDsSenha': self._password,
            'strRetorno': 'xml',
            'sCdMaoPropria': self._mao_propria,
            'nVlValorDeclarado': self._valor_declarado,
            'sCdAvisoRecebimento': self._aviso_recebimento,
            'nCdFormato': self._format,
            'sCepOrigem': self._zip_from,
            'sCepDestino': self._zip_to,
            'nCdServico': self._service,
            'nVlAltura': int(self._height),
            'nVlLargura': int(self._width),
            'nVlComprimento': int(self._length),
            'nVlDiametro': int(self._diameter),
            'nVlPeso': self._weight
        }

        return params

    def _make_request(self):
        params = self._get_parameters()
        url = '%s?%s' % (self._endpoint,
            urllib.urlencode(params))

        logger.debug('estimating freight on correios')
        logger.debug(params)

        data = urllib2.urlopen(url, timeout=5).read()

        """
        Exemplo do retorno:
            <cServico>
                <Codigo>40045</Codigo>
                <Valor>12,10</Valor>
                <PrazoEntrega>1</PrazoEntrega>
                <ValorMaoPropria>0,00</ValorMaoPropria>
                <ValorAvisoRecebimento>0,00</ValorAvisoRecebimento>
                <ValorValorDeclarado>1,00</ValorValorDeclarado>
                <EntregaDomiciliar>S</EntregaDomiciliar>
                <EntregaSabado>S</EntregaSabado>
                <Erro>0</Erro>
                <MsgErro></MsgErro>
            </cServico>
        """
        logger.debug('response from correios')
        logger.debug(data)

        response = simplexml.loads(data)

        result = response['Servicos']['cServico']

        erro = int(result.get('Erro'))
        if erro != 0:
            raise ValueError(result.get('MsgErro'))
        else:
            return result.get('Valor')

    def get_shipping_cost(self, bin, packages, country, state, zipcode):
        self._zip_to = zipcode

        total_cost = 0.0
        for pack in packages:
            # calc total weight, sum of all packages plus bin weight
            weight_total = sum([package.weight for package in pack]) + bin.weight

            # calc shipping price for each pack
            self._set_dimensions(bin, weight_total)

            price = self._make_request()
            total_cost += float(price.replace(',', '.'))

        return total_cost



# coding: utf-8
"""
package.py - shipping/cargo related calculations based on a unit of shipping (box, crate, package)

Created by Maximillian Dornseif on 2006-12-02.
Copyright HUDORA GmbH 2006, 2007, 2010
You might consider this BSD-Licensed.

https://github.com/hudora/pyShipping
"""


class Package(object):
    """Represents a package as used in cargo/shipping aplications."""

    def __init__(self, size, weight=0, nosort=False):
        """Generates a new Package object.

        The size can be given as an list of integers or an string where the sizes are
        separated by the letter 'x':
        >>> Package((300, 400, 500))
        <Package 500x400x300>
        >>> Package('300x400x500')
        <Package 500x400x300>
        """
        self.weight = weight
        if "x" in size:
            self.heigth, self.width, self.length = [int(x) for x in size.split('x')]
        else:
            self.heigth, self.width, self.length = size
        if not nosort:
            (self.heigth, self.width, self.length) = sorted((int(self.heigth), int(self.width),
                                                             int(self.length)), reverse=True)
        self.volume = self.heigth * self.width * self.length
        self.size = (self.heigth, self.width, self.length)

    def _get_gurtmass(self):
        """'gurtamss' is the circumference of the box plus the length - which is often used to
            calculate shipping costs.

            >>> Package((100,110,120)).gurtmass
            540
        """

        dimensions = (self.heigth, self.width, self.length)
        maxdimension = max(dimensions)
        otherdimensions = list(dimensions)
        del otherdimensions[otherdimensions.index(maxdimension)]
        return maxdimension + 2 * (sum(otherdimensions))
    gurtmass = property(_get_gurtmass)

    def hat_gleiche_seiten(self, other):
        """Prüft, ob other mindestens eine gleich grosse Seite mit self hat."""

        meineseiten = set([(self.heigth, self.width), (self.heigth, self.length), (self.width, self.length)])
        otherseiten = set([(other.heigth, other.width), (other.heigth, other.length),
                           (other.width, other.length)])
        return bool(meineseiten.intersection(otherseiten))

    def __getitem__(self, key):
        """The coordinates can be accessed as if the object is a tuple.
        >>> p = Package((500, 400, 300))
        >>> p[0]
        500
        """
        if key == 0:
            return self.heigth
        if key == 1:
            return self.width
        if key == 2:
            return self.length
        if isinstance(key, tuple):
            return (self.heigth, self.width, self.length)[key[0]:key[1]]
        if isinstance(key, slice):
            return (self.heigth, self.width, self.length)[key]
        raise IndexError

    def __contains__(self, other):
        """Checks if on package fits within an other.

        >>> Package((1600, 250, 480)) in Package((1600, 250, 480))
        True
        >>> Package((1600, 252, 480)) in Package((1600, 250, 480))
        False
        """
        return self[0] >= other[0] and self[1] >= other[1] and self[2] >= other[2]

    def __hash__(self):
        return self.heigth + (self.width << 16) + (self.length << 32)

    def __eq__(self, other):
        """Package objects are equal if they have exactly the same dimensions.

           Permutations of the dimensions are considered equal:

           >>> Package((100,110,120)) == Package((100,110,120))
           True
           >>> Package((120,110,100)) == Package((100,110,120))
           True
        """
        return (self.heigth == other.heigth and self.width == other.width and self.length == other.length)

    def __cmp__(self, other):
        """Enables to sort by Volume."""
        return cmp(self.volume, other.volume)

    def __mul__(self, multiplicand):
        """Package can be multiplied with an integer. This results in the Package beeing
           stacked along the biggest side.

           >>> Package((400,300,600)) * 2
           <Package 600x600x400>
           """
        if self.weight:
            new_weight = self.weight * multiplicand
        else:
            new_weight = None
        return Package((self.heigth, self.width, self.length * multiplicand), new_weight)

    def __add__(self, other):
        """
            >>> Package((1600, 250, 480)) + Package((1600, 470, 480))
            <Package 1600x720x480>
            >>> Package((1600, 250, 480)) + Package((1600, 480, 480))
            <Package 1600x730x480>
            >>> Package((1600, 250, 480)) + Package((1600, 490, 480))
            <Package 1600x740x480>
            """
        meineseiten = set([(self.heigth, self.width), (self.heigth, self.length),
                           (self.width, self.length)])
        otherseiten = set([(other.heigth, other.width), (other.heigth, other.length),
                           (other.width, other.length)])
        if not meineseiten.intersection(otherseiten):
            raise ValueError("%s has no fitting sites to %s" % (self, other))
        candidates = sorted(meineseiten.intersection(otherseiten), reverse=True)
        stack_on = candidates[0]
        mysides = [self.heigth, self.width, self.length]
        mysides.remove(stack_on[0])
        mysides.remove(stack_on[1])
        othersides = [other.heigth, other.width, other.length]
        othersides.remove(stack_on[0])
        othersides.remove(stack_on[1])

        if self.weight and other.weight:
            new_weight = self.weight + other.weight
        else:
            new_weight = None

        return Package((stack_on[0], stack_on[1], mysides[0] + othersides[0]), new_weight)

    def __str__(self):
        if self.weight:
            return "%dx%dx%d %dg" % (self.heigth, self.width, self.length, self.weight)
        else:
            return "%dx%dx%d" % (self.heigth, self.width, self.length)

    def __repr__(self):
        if self.weight:
            return "<Package %dx%dx%d %d>" % (self.heigth, self.width, self.length, self.weight)
        else:
            return "<Package %dx%dx%d>" % (self.heigth, self.width, self.length)
